@@ -11,7 +11,7 @@ use strict;
 use vars qw($VERSION);
 use Carp ();
 
-$VERSION = '1.30';
+$VERSION = '1.33';
 
 sub PV  { 0 }
 sub IV  { 1 }
@@ -81,6 +81,7 @@ my %def_attr = (
     keep_meta_info      => 0,
     allow_loose_quotes  => 0,
     allow_loose_escapes => 0,
+    allow_unquoted_escape => 0,
     allow_whitespace    => 0,
     chomp_verbatim      => 0,
     types               => undef,
@@ -91,6 +92,8 @@ my %def_attr = (
     quote_space         => 1,
     quote_null          => 1,
     quote_binary        => 1,
+    diag_verbose        => 0,
+    decode_utf8         => 1,
 
     _EOF                => 0,
     _RECNO              => 0,
@@ -352,9 +355,9 @@ sub _parse {
 
     return 0 if(!defined $line);
 
-    my ($binary, $quot, $sep, $esc, $types, $keep_meta_info, $allow_whitespace, $eol, $blank_is_undef, $empty_is_undef)
+    my ($binary, $quot, $sep, $esc, $types, $keep_meta_info, $allow_whitespace, $eol, $blank_is_undef, $empty_is_undef, $unquot_esc, $decode_utf8)
          = @{$self}{
-            qw/binary quote_char sep_char escape_char types keep_meta_info allow_whitespace eol blank_is_undef empty_is_undef/
+            qw/binary quote_char sep_char escape_char types keep_meta_info allow_whitespace eol blank_is_undef empty_is_undef allow_unquoted_escape decode_utf8/
            };
 
     $sep  = ',' unless (defined $sep);
@@ -369,15 +372,19 @@ sub _parse {
     my $re_split       = $self->{_re_split}->{$quot}->{$esc}->{$sep} ||= _make_regexp_split_column($esc, $quot, $sep);
     my $re_quoted       = $self->{_re_quoted}->{$quot}               ||= qr/^\Q$quot\E(.*)\Q$quot\E$/s;
     my $re_in_quot_esp1 = $self->{_re_in_quot_esp1}->{$esc}          ||= qr/\Q$esc\E(.)/;
-    my $re_in_quot_esp2 = $self->{_re_in_quot_esp2}->{$quot}->{$esc} ||= qr/[\Q$quot$esc\E0]/;
+    my $re_in_quot_esp2 = $self->{_re_in_quot_esp2}->{$quot}->{$esc} ||= qr/[\Q$quot$esc$sep\E0]/;
     my $re_quot_char    = $self->{_re_quot_char}->{$quot}            ||= qr/\Q$quot\E/;
-    my $re_esc          = $self->{_re_esc}->{$quot}->{$esc}          ||= qr/\Q$esc\E(\Q$quot\E|\Q$esc\E|0)/;
+    my $re_esc          = $self->{_re_esc}->{$quot}->{$esc}          ||= qr/\Q$esc\E(\Q$quot\E|\Q$esc\E|\Q$sep\E|0)/;
     my $re_invalid_quot = $self->{_re_invalid_quot}->{$quot}->{$esc} ||= qr/^$re_quot_char|[^\Q$re_esc\E]$re_quot_char/;
 
     if ($allow_whitespace) {
         $re_split = $self->{_re_split_allow_sp}->{$quot}->{$esc}->{$sep}
                      ||= _make_regexp_split_column_allow_sp($esc, $quot, $sep);
     }
+    if ($unquot_esc) {
+        $re_split = $self->{_re_split_allow_unqout_esc}->{$quot}->{$esc}->{$sep}
+                     ||= _make_regexp_split_column_allow_unqout_esc($esc, $quot, $sep);
+    }
 
     my $palatable = 1;
     my @part      = ();
@@ -403,7 +410,7 @@ sub _parse {
 
     my $pos = 0;
 
-    my $utf8 = 1 if utf8::is_utf8( $line ); # if UTF8 marked, flag on.
+    my $utf8 = 1 if $decode_utf8 and utf8::is_utf8( $line ); # if decode_utf8 is true(default) and UTF8 marked, flag on.
 
     for my $col ( $line =~ /$re_split/g ) {
 
@@ -457,7 +464,8 @@ sub _parse {
                         $palatable = 0;
                         last;
                     }
-                    else {
+
+                    unless ($self->{allow_loose_quotes}) {
                         $col =~ s/\Q$esc\E(.)/$1/g;
                     }
                 }
@@ -534,9 +542,14 @@ sub _parse {
                 $col = undef;
             }
 
+            if ( $unquot_esc ) {
+                $col =~ s/\Q$esc\E(.)/$1/g;
+            }
+
         }
 
-        if ( $binary && defined $col && _is_valid_utf8($col) ) {
+        utf8::encode($col) if $utf8;
+        if ( $decode_utf8 && defined $col && _is_valid_utf8($col) ) {
             utf8::decode($col);
         }
 
@@ -554,6 +567,25 @@ sub _parse {
     if ($palatable) {
         $self->{_ERROR_INPUT} = undef;
         $self->{_FIELDS}      = \@part;
+
+        if ( $self->{_BOUND_COLUMNS} ) {
+            my @vals  = @part;
+            my ( $max, $count ) = ( scalar @vals, 0 );
+
+            if ( @{ $self->{_BOUND_COLUMNS} } < $max ) {
+                $self->_set_error_diag(3006);
+                return;
+            }
+
+            for ( my $i = 0; $i < $max; $i++ ) {
+                my $bind = $self->{_BOUND_COLUMNS}->[ $i ];
+                if ( Scalar::Util::readonly( $$bind ) ) {
+                    $self->_set_error_diag(3008);
+                    return;
+                }
+                $$bind = $vals[ $i ];
+            }
+        }
     }
 
     $self->{_FFLAGS} = $keep_meta_info ? $meta_flag : [];
@@ -569,11 +601,36 @@ sub _make_regexp_split_column {
         return qr/([^\Q$sep\E]*)\Q$sep\E/s;
     }
 
-   qr/(
+   return qr/(
         \Q$quot\E
             [^\Q$quot$esc\E]*(?:\Q$esc\E[\Q$quot$esc\E0][^\Q$quot$esc\E]*)*
         \Q$quot\E
         | # or
+        \Q$quot\E
+            (?:\Q$esc\E[\Q$quot$esc$sep\E0]|[^\Q$quot$esc$sep\E])*
+        \Q$quot\E
+        | # or
+        [^\Q$sep\E]*
+       )
+       \Q$sep\E
+    /xs;
+}
+
+
+sub _make_regexp_split_column_allow_unqout_esc {
+    my ($esc, $quot, $sep) = @_;
+
+   return qr/(
+        \Q$quot\E
+            [^\Q$quot$esc\E]*(?:\Q$esc\E[\Q$quot$esc\E0][^\Q$quot$esc\E]*)*
+        \Q$quot\E
+        | # or
+        \Q$quot\E
+            (?:\Q$esc\E[\Q$quot$esc$sep\E0]|[^\Q$quot$esc$sep\E])*
+        \Q$quot\E
+        | # or
+            (?:\Q$esc\E[\Q$quot$esc$sep\E0]|[^\Q$quot$esc$sep\E])*
+        | # or
         [^\Q$sep\E]*
        )
        \Q$sep\E
@@ -598,7 +655,7 @@ sub _make_regexp_split_column_allow_sp {
     qr/$ws*
        (
         \Q$quot\E
-            [^\Q$quot$esc\E]*(?:\Q$esc\E[\Q$quot$esc\E0][^\Q$quot$esc\E]*)*
+            [^\Q$quot$esc\E]*(?:\Q$esc\E[\Q$quot\E][^\Q$quot$esc\E]*)*
         \Q$quot\E
         | # or
         [^\Q$sep\E]*?
@@ -639,7 +696,7 @@ sub getline {
 
     require IO::Handle;
 
-    $self->{_EOF} = eof($io) ? 1 : '';
+    $self->{_EOF} = $io->eof ? 1 : '';
 
     my $quot = $self->{quote_char};
     my $sep  = $self->{sep_char};
@@ -663,7 +720,10 @@ sub getline {
         LOOP: {
             my $is_continued   = scalar(my @list = $line =~ /$re/g) % 2; # if line is valid, quot is even
 
-            if ( $line =~ /${re}0/ ) { # null suspicion case
+            if ( $self->{allow_loose_quotes } ) {
+                $is_continued = 0;
+            }
+            elsif ( $line =~ /${re}0/ ) { # null suspicion case
                 $is_continued = $line =~ qr/
                     ^
                     (
@@ -684,7 +744,7 @@ sub getline {
                 /x ? 0 : 1;
             }
 
-            if ( $is_continued and !eof($io) ) {
+            if ( $is_continued and !$io->eof) {
                 $line .= $io->getline();
                 goto LOOP;
             }
@@ -695,38 +755,13 @@ sub getline {
 
     $self->_parse($line);
 
-    return $self->_return_getline_result();
-}
-
-
-sub _return_getline_result {
-
     if ( eof ) {
-        $_[0]->{_AUTO_DETECT_CR} = 0;
+        $self->{_AUTO_DETECT_CR} = 0;
     }
 
-    return unless $_[0]->{_STATUS};
+    return unless $self->{_STATUS};
 
-    return [ $_[0]->_fields() ] unless $_[0]->{_BOUND_COLUMNS};
-
-    my @vals  = $_[0]->_fields();
-    my ( $max, $count ) = ( scalar @vals, 0 );
-
-    if ( @{ $_[0]->{_BOUND_COLUMNS} } < $max ) {
-            $_[0]->_set_error_diag(3006);
-            return;
-    }
-
-    for ( my $i = 0; $i < $max; $i++ ) {
-        my $bind = $_[0]->{_BOUND_COLUMNS}->[ $i ];
-        if ( Scalar::Util::readonly( $$bind ) ) {
-            $_[0]->_set_error_diag(3008);
-            return;
-        }
-        $$bind = $vals[ $i ];
-    }
-
-    return [];
+    return $self->{_BOUND_COLUMNS} ? [] : [ $self->_fields() ];
 }
 ################################################################################
 # getline_all
@@ -746,7 +781,7 @@ sub getline_all {
 
     while ( my $row = $self->getline($io) ) {
         next if $offset && $offset-- > 0;               # skip
-        last if defined $len && !$tail && $n >= $len;   # exceedes limit size
+        last if defined $len && !$tail && $n >= $len;   # exceeds limit size
         push @list, $row;
         ++$n;
         if ( $tail && $n > $tail ) {
@@ -893,7 +928,7 @@ sub is_missing {
 }
 ################################################################################
 # _check_type
-#  take an arg as scalar referrence.
+#  take an arg as scalar reference.
 #  if not numeric, make the value 0. otherwise INTEGERized.
 ################################################################################
 sub _check_type {
@@ -930,7 +965,7 @@ sub _set_error_diag {
 BEGIN {
     for my $method ( qw/always_quote binary keep_meta_info allow_loose_quotes allow_loose_escapes
                             verbatim blank_is_undef empty_is_undef quote_space quote_null
-                            quote_binary/ ) {
+                            quote_binary allow_unquoted_escape/ ) {
         eval qq|
             sub $method {
                 \$_[0]->{$method} = defined \$_[1] ? \$_[1] : 0 if (\@_ > 1);
@@ -952,6 +987,15 @@ sub sep_char {
     $self->{sep_char};
 }
 
+sub decode_utf8 {
+    my $self = shift;
+    if ( @_ ) {
+        $self->{decode_utf8} = $_[0];
+        my $ec = _check_sanity( $self );
+        $ec and Carp::croak( $self->SetDiag( $ec ) );
+    }
+    $self->{decode_utf8};
+}
 
 sub quote_char {
     my $self = shift;
@@ -1017,6 +1061,17 @@ sub auto_diag {
     $self->{auto_diag};
 }
 
+sub diag_verbose {
+    my $self = shift;
+    if (@_) {
+        my $v = shift;
+        !defined $v || $v eq "" and $v = 0;
+        $v =~ m/^[0-9]/ or $v = $v ? 1 : 0; # default for true/false
+        $self->{diag_verbose} = $v;
+    }
+    $self->{diag_verbose};
+}
+
 sub _is_valid_utf8 {
     return ( $_[0] =~ /^(?:
          [\x00-\x7F]
@@ -1101,7 +1156,7 @@ is a XS module and Text::CSV_PP is a Puer Perl one.
 
 =head1 VERSION
 
-    1.30
+    1.31
 
 This module is compatible with Text::CSV_XS B<0.99>.
 (except for diag_verbose and allow_unquoted_escape)
@@ -1218,7 +1273,7 @@ is read as
 
  (1, undef, undef, " ", 2)
 
-Note that this only effects fields that are I<realy> empty, not fields
+Note that this only effects fields that are I<really> empty, not fields
 that are empty after stripping allowed whitespace. YMMV.
 
 =item quote_char
@@ -1562,6 +1617,17 @@ Will set C<$hr->{"\cAUNDEF\cA"}> to the 1st field, C<$hr->{""}> to the
 
 C<column_names ()> croaks on invalid arguments.
 
+=head2 print_hr
+
+ $csv->print_hr ($io, $ref);
+
+Provides an easy way to print a C<$ref> as fetched with L<getline_hr>
+provided the column names are set with L<column_names>.
+
+It is just a wrapper method with basic parameter checks over
+
+ $csv->print ($io, [ map { $ref->{$_} } $csv->column_names ]);
+
 =head2 bind_columns
 
 Takes a list of references to scalars to store the fields fetched
@@ -1704,7 +1770,7 @@ C<combine ()> or C<parse ()>, whichever was called more recently.
  $error_str    = "" . $csv->error_diag ();
  ($cde, $str, $pos) = $csv->error_diag ();
 
-If (and only if) an error occured, this function returns the diagnostics
+If (and only if) an error occurred, this function returns the diagnostics
 of that error.
 
 If called in void context, it will print the internal error code and the
@@ -1732,10 +1798,10 @@ Use to reset the diagnostics if you are dealing with errors.
 
 =head1 DIAGNOSTICS
 
-If an error occured, $csv->error_diag () can be used to get more information
+If an error occurred, $csv->error_diag () can be used to get more information
 on the cause of the failure. Note that for speed reasons, the internal value
 is never cleared on success, so using the value returned by error_diag () in
-normal cases - when no error occured - may cause unexpected results.
+normal cases - when no error occurred - may cause unexpected results.
 
 Note: CSV_PP's diagnostics is different from CSV_XS's:
 
@@ -1824,7 +1890,7 @@ Text::CSV was written by E<lt>alan[at]mfgrtl.comE<gt>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2005-2013 by Makamaka Hannyaharamitu, E<lt>makamaka[at]cpan.orgE<gt>
+Copyright 2005-2015 by Makamaka Hannyaharamitu, E<lt>makamaka[at]cpan.orgE<gt>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
